/***************************************************************************
 *   Copyright (C) 2003 by Konstantin Sobolev                              *
 *   konstantin.sobolev@gmail.com                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <klocale.h>
#include <kcolordialog.h>
#include <kinputdialog.h>
#include <kmessagebox.h>
#include <kparts/part.h>
#include <kparts/componentfactory.h>
#include <kregexpeditorinterface.h>
#include "klogsource.h"
#include "kfilelogsource.h"
#include "kprocessoutputlogsource.h"

KLogSource::KLogSource(QString name, QColor textColor, bool bold, bool underline, bool italic): name(name), textColor(textColor), bold(bold), underline(underline), italic(italic), enabled(TRUE), showingAlert(FALSE) {}


KLogSource::~KLogSource() {}

//factory method
KLogSource* KLogSource::createFromConfig(KConfig* config) {
    //read type, construct instance,
    //call instance->readConfig
    LogSourceType type = (LogSourceType)config->readNumEntry("Type");
    switch( type ) {
    case KLogSource::File: {
            KLogSource* res = new KFileLogSource();
            res->readConfig(config);
            return res;
        }
    case KLogSource::Process: {
            KProcessOutputLogSource* res = new KProcessOutputLogSource();
            res->readConfig(config);
            return res;
        }
    default:
        printf ("Unknown log source type: %d\n", type);
    }
    return NULL;
}

/*!
    \fn KLogSource::writeConfig(KConfig* config)
 */
void KLogSource::writeConfig( KConfig* config) {
    config->writeEntry("Type",(int)getType());
    config->writeEntry("Name",name);
    config->writeEntry("Bold",isBold());
    config->writeEntry("Italic",isItalic());
    config->writeEntry("Underline",isUnderline());
    config->writeEntry("TextColor",getTextColor());
    config->writeEntry("Enabled", isEnabled());
    filters.writeConfig(config, name);    
}

/*!
    \fn KLogSource::readConfig(KConfig* config)
 */
void KLogSource::readConfig( KConfig* config) {
    name=config->readEntryUntranslated("Name");
    bold=config->readBoolEntry("Bold");
    italic=config->readBoolEntry("Italic");
    underline=config->readBoolEntry("Underline");
    textColor=config->readColorEntry("TextColor");
    enabled=config->readBoolEntry("Enabled", true);
    
    filters.readConfig(config, name);

    showingAlert=FALSE;
    resetDontShowAlertAgain();
    emit styleChanged(this);
}

void KLogSource::insertMenuItems(QPopupMenu* popup) {
    this->popup = popup;
    popup->insertTearOffHandle();

    enabledId = popup->insertItem(i18n("Enabled"), this, SLOT(changeEnabled()));
    popup->insertSeparator();
    boldId = popup->insertItem(i18n("Bold"), this, SLOT(changeBold()));
    italicId = popup->insertItem(i18n("Italic"), this, SLOT(changeItalic()));
    underlineId = popup->insertItem(i18n("Underline"), this, SLOT(changeUnderline()));

    popup->setItemChecked(enabledId, isEnabled());
    popup->setItemChecked(boldId, isBold());
    popup->setItemChecked(italicId, isItalic());
    popup->setItemChecked(underlineId, isUnderline());

    popup->insertItem(i18n("Color"), this, SLOT(changeColor()));

    popup->insertSeparator();
    popup->insertItem(i18n("Filters"), this, SLOT(changeFilters()));
}

/*!
    \fn KLogSource::changeBold()
 */
void KLogSource::changeBold() {    
    setBold(!isBold());    
    popup->setItemChecked(boldId, isBold());   
}

/*!
    \fn KLogSource::changeColor()
 */
void KLogSource::changeColor() {
    QColor c=getTextColor();
    int res = KColorDialog::getColor(c, NULL);
    if (res == KColorDialog::Accepted)
        setTextColor(c);
}

/*!
    \fn KLogSource::changeItalic()
 */
void KLogSource::changeItalic() {
    setItalic(!isItalic());
    popup->setItemChecked(italicId, isItalic());   
}

/*!
    \fn KLogSource::changeUnderline()
 */
void KLogSource::changeUnderline() {
    setUnderline(!isUnderline());
    popup->setItemChecked(underlineId, isUnderline());   
}

void KLogSource::changeFilters() {
    filters.edit(NULL);
}

/*!
    \fn KLogSource::changeEnabled()
 */
void KLogSource::changeEnabled() {
    setEnabled(!isEnabled());
    popup->setItemChecked(enabledId, isEnabled());   
}

void KLogSource::sendLineAdded(KLogSource* source, const QString& line) {
    if (!isEnabled())
        return;

    if (filters.passes(line))
        emit lineAdded(source, line);

    QString alert = filters.alert(line);
    if (!showingAlert && alert != QString::null){
        showingAlert=TRUE;        
        KMessageBox::information(
            NULL, 
            i18n("%1 triggered an alert:\n%2\nline:\n%3").arg(source->getName()).arg(alert).arg(line), 
            "KLogView", 
            name
        );
        showingAlert=FALSE;
    }
}

void KLogSource::resetDontShowAlertAgain(){
    KConfig *config = 0;
    QString grpNotifMsgs = QString::fromLatin1("Notification Messages");
    config = KGlobal::config();
    KConfigGroupSaver saver( config, grpNotifMsgs );
    config->writeEntry(name, true);
    config->sync();
}

#include "klogsource.moc"
