/*
 * Copyright (C) 2003 Konstantin Sobolev <konstantin.sobolev@gmail.com>
 */

#include "klogview.h"

#include <qlabel.h>
#include <qcursor.h>
#include <qtimer.h>

#include <kapp.h>
#include <kmainwindow.h>
#include <klocale.h>
#include <ktextedit.h>
#include <kinputdialog.h>
#include <kglobal.h>
#include <kiconloader.h>

KLogView::KLogView(bool startMinimized)
        : KDockMainWindow( 0, "klogview" ), startMinimized(startMinimized) {
    // set the shell's ui resource file
    setXMLFile("klogviewui.rc");

    KIconLoader &loader= *KGlobal::iconLoader();
    smallIcon = new QPixmap(loader.loadIcon("klogview", KIcon::Small));

    popup = new KPopupMenu(this);
    popup->insertItem(i18n("New log panel"), this, SLOT(addLogWindow()));
    popup->insertSeparator();
    readConfig();
    
    connect(kapp, SIGNAL(shutDown()), this, SLOT(shutDown()));

    KLogTray* tray = new KLogTray(this);
    tray->show();

    if (startMinimized)
        QTimer::singleShot(20, tray, SLOT(minimizeRestoreLogView()));
}

KLogView::~KLogView() {
    delete smallIcon;
    delete popup;
}

void KLogView::mousePressEvent(QMouseEvent *e) {
    if(e->button() == RightButton)
        popup->popup(QCursor::pos());
}


/*!
    \fn KLogView::addLogWindow()
 */
void KLogView::addLogWindow() {
    bool ok;
    QString name = KInputDialog::getText(i18n("Log panel name"), i18n("Log panel name: "), QString::null, &ok, this);
    if (!ok)
        return;

    addLogWindow(name);
}

KLogWidget* KLogView::addLogWindow(QString name) {
    KDockWidget* dockWidget = createDockWidget(name, *smallIcon, 0L, name);
    KLogWidget* logWidget = new KLogWidget(dockWidget, name);
    logWindows.append(logWidget);
    connect(dockWidget, SIGNAL(headerCloseButtonClicked()), this, SLOT(closeLogWindow()));
    //connect(dockWidget, SIGNAL(iMBeingClosed()), this, SLOT(closeLogWindow()));

    logWidget->setLogView(this);
    dockWidget->setWidget(logWidget);
    KDockWidget::DockPosition position = KDockWidget::DockCorner; // KDockWidget::DockFullSite KDockWidget::DockFullDocking
    dockWidget->setEnableDocking(position);
    dockWidget->setDockSite(position);
    //dockWidget->setDockWindowTransient(this,true);

    if (getMainDockWidget()==NULL) {
        setView(dockWidget);
        setMainDockWidget(dockWidget);
    } else {
        dockWidget->manualDock (getMainDockWidget(), KDockWidget::DockBottom);
        //dockWidget->toDesktop();
    }
    dockWidget->show();

    return logWidget;
}

/*!
    \fn KLogView::closeLogWindow()
 */
void KLogView::closeLogWindow() {
    const QObject* s = sender();
    if (s == NULL || strcmp(s->className(),"KDockWidget"))
        return;
    KDockWidget* sender = (KDockWidget*)s;
    logWindows.remove((KLogWidget*)(sender->getWidget()));
}

void KLogView::writeConfig() {
    KConfig* conf = kapp->config();

    conf->setGroup("Global");
    conf->writeEntry("Geometry", QRect(pos(), size()));

    conf->setGroup("LogWindows");
    conf->writeEntry("NumberOfWindows",logWindows.count());
    KLogWidget* w;
    int i=1;
    for (w=logWindows.first();w;w=logWindows.next(),i++)
        conf->writeEntry("GroupName"+QString::number(i), w->name());

    for (w=logWindows.first();w;w=logWindows.next()) {
        conf->setGroup(w->name());
        w->writeConig(conf);
    }

    writeDockConfig(conf, "DockConfig");
}

void KLogView::readConfig() {
    KConfig* conf = kapp->config();

    conf->setGroup("Global");
    QRect geometry = conf->readRectEntry ("Geometry");
    if (geometry.isNull())
        return;
    resize (geometry.size());
    move (geometry.topLeft());

    conf->setGroup("LogWindows");
    int nWindows = conf->readNumEntry("NumberOfWindows");
    QString* groupNames = new QString[nWindows];
    for (int i=1;i<=nWindows;i++)
        groupNames[i-1]=conf->readEntryUntranslated("GroupName"+QString::number(i));

    for (int i=0;i<nWindows;i++) {
        QString name =groupNames[i];
        KLogWidget* w = addLogWindow(name);
        conf->setGroup(name);
        w->readConfig(conf);
    }
    delete[] groupNames;

    readDockConfig(conf, "DockConfig");
}

void KLogView::closeEvent(QCloseEvent* event) {
    writeConfig();
    KDockMainWindow::closeEvent(event);
}

/*!
    \fn KLogView::shutDown()
 */
void KLogView::shutDown() {
    writeConfig();
}

#include "klogview.moc"
