/***************************************************************************
 *   Copyright (C) 2003 by Konstantin Sobolev                              *
 *   konstantin.sobolev@gmail.com                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <kfontdialog.h>
#include <kiconloader.h>
#include <kinputdialog.h>
#include <qtooltip.h>
#include "klogwidget.h"
#include "kprocessoutputlogsource.h"

static int counter=0;

KLogWidget::KLogWidget(QWidget* parent, const char* name) : KTextEdit("", QString::null, parent, name) {
    setTextFormat(LogText);
    font = currentFont();

    tags.setAutoDelete(TRUE);
    logSources.setAutoDelete(TRUE);
    QToolTip::add(this, name);
}

KLogWidget::~KLogWidget() {}

/*!
    \fn KLogWidget::appendLogLine(KLogSource* source,const QString& line)
 */
void KLogWidget::appendLogLine(KLogSource* source,const QString& line) {
    appendLogLineMutex.lock();
    QString* tagName=tags[source->getName()];
    Q_ASSERT(tagName);
    append("<"+(*tagName)+">"+line+"<"+(*tagName)+"/>");
    //append("&lt;"+(*tagName)+"&gt;"+line+"&lt;"+(*tagName)+"/&gt;");
    //scrollToBottom();
    appendLogLineMutex.unlock();
}

void KLogWidget::addLogSource(KLogSource* source) {
    logSources.append(source);
    updateStyleTag(source);
    connect(source, SIGNAL(lineAdded(KLogSource*, const QString& )), this, SLOT(appendLogLine(KLogSource*, const QString& )));
    connect(source, SIGNAL(styleChanged(KLogSource* )), this, SLOT(updateStyleTag(KLogSource* )));
}

void KLogWidget::removeLogSource(KLogSource* source) {
    logSources.remove(source);
    tags.remove(source->getName());
    disconnect(source, 0, this, 0);
}

void KLogWidget::updateStyleTag(KLogSource* source) {
    QString name = source->getName();
    QString* tagName=tags[name];
    if (!tagName) {
        tagName = new QString(QString::number(counter++));
        tags.replace(name, tagName);
    }
    QStyleSheetItem* item = new QStyleSheetItem(styleSheet(), *tagName);
    item->setColor(source->getTextColor());
    item->setFontWeight(source->isBold()?QFont::Bold:QFont::Normal);
    item->setFontItalic(source->isItalic());
    item->setFontUnderline(source->isUnderline());
    updateContents();
}

QPopupMenu* KLogWidget::createPopupMenu(const QPoint& pos) {
    QPopupMenu* popup = KTextEdit::createPopupMenu(pos);
    popup->insertSeparator();
    insertMenuItems(popup);
    return popup;
}

void KLogWidget::insertMenuItems(QPopupMenu* popup) {
    KIconLoader &loader= *KGlobal::iconLoader();
    QPixmap clearIcon = loader.loadIcon("editclear", KIcon::Small);
    QPixmap fontIcon = loader.loadIcon("charset", KIcon::Small);

    if (logView)
        popup->insertItem(i18n("New log panel"), logView, SLOT(addLogWindow()));
    popup->insertItem(i18n("New file log"), this, SLOT(addFileLogSource()));
    popup->insertItem(i18n("New process output log"), this, SLOT(addProcessOutputLogSource()));
    popup->insertSeparator();
    popup->insertItem(QIconSet(clearIcon), i18n("Clear"), this, SLOT(clear()));
    popup->insertItem(QIconSet(fontIcon), i18n("Change font"), this, SLOT(changeFont()));
    if (!logSources.isEmpty()) {
        popup->insertSeparator();
        KLogSource* s;
        int idx;
        for (s=logSources.first(), idx=0;s;s=logSources.next(), idx++) {
            QPixmap p(16,16);
            p.fill(s->getTextColor());
            popup->insertItem(QIconSet(p), s->getName(), buildLogSourceMenu(s, idx));
        }
    }
}

QPopupMenu* KLogWidget::buildLogSourceMenu(KLogSource* source, int index) {
    QPopupMenu* m = new QPopupMenu(0L, source->getName());
    m->setCaption(source->getName());

    source->insertMenuItems(m);
    m->insertSeparator();

    KIconLoader &loader= *KGlobal::iconLoader();
    QPixmap pixmap = loader.loadIcon("fileclose", KIcon::Small);
    deleteItemId = m->insertItem(QIconSet(pixmap), i18n("Delete"),44);
    m->setItemParameter(deleteItemId, index);

    connect(m, SIGNAL(activated(int)), this, SLOT(menuActivated(int)));
    return m;
}

void KLogWidget::menuActivated(int id) {
    if (strcmp(sender()->className(),"QPopupMenu"))
        return;
    if (id!=deleteItemId)
        return;
    QPopupMenu* m = (QPopupMenu*)sender();
    int idx = m->itemParameter(id);
    KLogSource* s = logSources.at(idx);
    removeLogSource(s);
}


/*!
    \fn KLogWidget::changeFont()
 */
void KLogWidget::changeFont() {
    QFont fnt = font;
    int res = KFontDialog::getFont(fnt, false, this);
    if (res==KFontDialog::Accepted) {
        setFont(fnt);
        font=fnt;
    }
}

/*!
    \fn KLogWidget::writeConig(KConfig* config)
 */
void KLogWidget::writeConig(KConfig* config) {
    config->writeEntry("Font", font);
    config->writeEntry("NumberOfSources", logSources.count());
    KLogSource* s;
    int i=1;
    QString prefix = QString(name())+".";
    for (s=logSources.first();s;s=logSources.next(),i++)
        config->writeEntry("GroupName"+QString::number(i), prefix+s->getName());

    for (s=logSources.first();s;s=logSources.next()) {
        config->setGroup(prefix+s->getName());
        s->writeConfig(config);
    }
}


/*!
    \fn KLogWidget::readConfig(KConfig* config)
 */
void KLogWidget::readConfig(KConfig* config) {
    //setCurrentFont(config->readFontEntry("Font"));
    font=config->readFontEntry("Font");
    setFont(font);
    int nSources = config->readNumEntry("NumberOfSources");
    QString* groupNames = new QString[nSources];
    for (int i=1;i<=nSources;i++)
        groupNames[i-1]=config->readEntryUntranslated("GroupName"+QString::number(i));

    for (int i=0;i<nSources;i++) {
        config->setGroup(groupNames[i]);
        addLogSource(KLogSource::createFromConfig(config));
    }
    delete[] groupNames;
}

/*!
    \fn KLogWidget::addFileLogSource()
 */
void KLogWidget::addFileLogSource() {
    QString fileName = KFileDialog::getOpenFileName();
    if (fileName==QString::null) //???or size=0?
        return;
    addLogSource(new KFileLogSource(fileName));
}

/*!
    \fn KLogWidget::addProcessOutputLogSource()
 */
void KLogWidget::addProcessOutputLogSource() {
    bool ok;
    QString cmd = KInputDialog::getText(i18n("Command"), i18n("Command: "), QString::null, &ok, this);
    if (!ok)
        return;
    addLogSource(new KProcessOutputLogSource(cmd));
}


#include "klogwidget.moc"
